@@ -3,19 +3,32 @@
 A small and flexible Python web application framework.
 """
 
-__version__ = '3.5'
+# Should match setup.py VERSION
+__version__ = '3.6'
 
 # These are frequently needed by Quixote applications.
-from quixote.publish import \
-     get_publisher, get_request, get_response, get_path, redirect, \
-     get_session, get_session_manager, get_user, get_field, get_cookie, \
-     get_wsgi_app, cleanup, get_param
+from quixote.publish import (  # noqa: F401
+    get_publisher,
+    get_request,
+    get_response,
+    get_path,
+    redirect,
+    get_session,
+    get_session_manager,
+    get_user,
+    get_field,
+    get_cookie,
+    get_wsgi_app,
+    cleanup,
+    get_param,
+)
 
 
 # This is the default charset used by the HTTPRequest, HTTPResponse,
 # DefaultLogger, and sendmail components.
 DEFAULT_CHARSET = 'utf-8'
 
+
 def enable_ptl():
     """
     Installs the import hooks needed to import PTL modules.  This must
@@ -26,4 +39,4 @@ def enable_ptl():
     that, if you use ZODB, you must import ZODB before calling this
     function.
     """
-    import quixote.ptl.install
+    import quixote.ptl.install  # noqa: F401
